package mining;

import jason.environment.grid.GridWorldModel;
import jason.environment.grid.Location;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import java.io.*;
import java.util.*;

import mining.MiningPlanet.Move;

public class WorldModel extends GridWorldModel {

    public static final int   GOLD  = 16;
    public static final int   DEPOT = 32;
    public static final int   ENEMY = 64;

    Location                  depot;
    Set<Integer>              agWithGold;  // which agent is carrying gold
    int                       goldsInDepot   = 0;
    int                       initialNbGolds = 0;

    private Logger            logger   = Logger.getLogger("jasonTeamSimLocal.mas2j." + WorldModel.class.getName());

    private String            id = "WorldModel";
    
    // singleton pattern
    protected static WorldModel model = null;
    
    synchronized public static WorldModel create(int w, int h, int nbAgs) {
        if (model == null) {
            model = new WorldModel(w, h, nbAgs);
        }
        return model;
    }
    
    public static WorldModel get() {
        return model;
    }
    
    public static void destroy() {
        model = null;
    }

    private WorldModel(int w, int h, int nbAgs) {
        super(w, h, nbAgs);
        agWithGold = new HashSet<Integer>();
    }

    public String getId() {
        return id;
    }
    public void setId(String id) {
        this.id = id;
    }
    public String toString() {
        return id;
    }
    
    public Location getDepot() {
        return depot;
    }

    public int getGoldsInDepot() {
        return goldsInDepot;
    }
    
    public boolean isAllGoldsCollected() {
        return goldsInDepot == initialNbGolds;
    }
    
    public void setInitialNbGolds(int i) {
        initialNbGolds = i;
    }
    
    public int getInitialNbGolds() {
        return initialNbGolds;
    }

    public boolean isCarryingGold(int ag) {
        return agWithGold.contains(ag);
    }

    public void setDepot(int x, int y) {
        depot = new Location(x, y);
        data[x][y] = DEPOT;
    }

    public void setAgCarryingGold(int ag) {
        agWithGold.add(ag);
    }
    public void setAgNotCarryingGold(int ag) {
        agWithGold.remove(ag);
    }

    /** Actions **/

    boolean move(Move dir, int ag) throws Exception {
        Location l = getAgPos(ag);
        switch (dir) {
        case UP:
            if (isFree(l.x, l.y - 1)) {
                setAgPos(ag, l.x, l.y - 1);
            }
            break;
        case DOWN:
            if (isFree(l.x, l.y + 1)) {
                setAgPos(ag, l.x, l.y + 1);
            }
            break;
        case RIGHT:
            if (isFree(l.x + 1, l.y)) {
                setAgPos(ag, l.x + 1, l.y);
            }
            break;
        case LEFT:
            if (isFree(l.x - 1, l.y)) {
                setAgPos(ag, l.x - 1, l.y);
            }
            break;
        }
        return true;
    }

    boolean pick(int ag) {
        Location l = getAgPos(ag);
        if (hasObject(WorldModel.GOLD, l.x, l.y)) {
            if (!isCarryingGold(ag)) {
            	if (hasFriendToHelpNearby(l.x, l.y)) {
	                remove(WorldModel.GOLD, l.x, l.y);
	                setAgCarryingGold(ag);
	                return true;
            	} else {
            		logger.warning("Agent " + (ag + 1) + " is trying the pick gold, but it needs another agent in the neighboring row to help!");
            	}
            } else {
                logger.warning("Agent " + (ag + 1) + " is trying the pick gold, but it is already carrying gold!");
            }
        } else {
            logger.warning("Agent " + (ag + 1) + " is trying the pick gold, but there is no gold at " + l.x + "x" + l.y + "!");
        }
        return false;
    }

	private boolean hasFriendToHelpNearby(int x, int y) {
		if (inGrid(x+1, y) && hasObject(WorldModel.AGENT, x+1,y)) {
			return true;
		}

		if (inGrid(x-1, y) && hasObject(WorldModel.AGENT, x-1,y)) {
			return true;
		}
		
		if (inGrid(x, y-1) && hasObject(WorldModel.AGENT, x,y-1)) {
			return true;
		}
		
		if (inGrid(x, y+1) && hasObject(WorldModel.AGENT, x,y+1)) {
			return true;
		}
		
		return false;
	}

	boolean drop(int ag) {
        Location l = getAgPos(ag);
        if (isCarryingGold(ag)) {
            //if (l.equals(getDepot())) {
			if (hasObject(WorldModel.DEPOT, l.x, l.y)) {
                goldsInDepot++;
                logger.info("Agent " + (ag + 1) + " carried a gold to depot!");
            } else {
                add(WorldModel.GOLD, l.x, l.y);
            }
            setAgNotCarryingGold(ag);
            return true;
        }
        return false;
    }

    /*
    public void clearAgView(int agId) {
        clearAgView(getAgPos(agId).x, getAgPos(agId).y);
    }

    public void clearAgView(int x, int y) {
        int e1 = ~(ENEMY + ALLY + GOLD);
        if (x > 0 && y > 0) {
            data[x - 1][y - 1] &= e1;
        } // nw
        if (y > 0) {
            data[x][y - 1] &= e1;
        } // n
        if (x < (width - 1) && y > 0) {
            data[x + 1][y - 1] &= e1;
        } // ne

        if (x > 0) {
            data[x - 1][y] &= e1;
        } // w
        data[x][y] &= e1; // cur
        if (x < (width - 1)) {
            data[x + 1][y] &= e1;
        } // e

        if (x > 0 && y < (height - 1)) {
            data[x - 1][y + 1] &= e1;
        } // sw
        if (y < (height - 1)) {
            data[x][y + 1] &= e1;
        } // s
        if (x < (width - 1) && y < (height - 1)) {
            data[x + 1][y + 1] &= e1;
        } // se
    }
    */
	
	static WorldModel loadWorld(File file) throws Exception {
		Scanner sc = new Scanner(new FileInputStream(file));
		ArrayList<String> lines = new ArrayList<String>();
		while(sc.hasNextLine()) lines.add(sc.nextLine());
		
		int width = lines.get(0).length();
		int height = lines.size();
		
		WorldModel model = WorldModel.create(width, height, 4);
		int agCounter = 0;
		for(int y = 0; y < height; y++) {
			for(int x = 0; x < width; x++) {
				switch(lines.get(y).charAt(x)) {
					case 'D':
						model.add(WorldModel.DEPOT, x, y); break;
					case 'X':
						model.add(WorldModel.OBSTACLE, x, y); break;
					case 'G':
						model.add(WorldModel.GOLD, x, y); break;
					case 'A':
						model.setAgPos(agCounter++, x, y); break;
					default:
				}
			}
		}
		model.setInitialNbGolds(model.countObjects(WorldModel.GOLD));
		return model;
	}
	
	static WorldModel world0() throws Exception {
		WorldModel model = WorldModel.create(12, 12, 4);
		model.setAgPos(0, 1, 0);
        model.setAgPos(1, 11, 0);
        model.setAgPos(2, 3, 11);
        model.setAgPos(3, 11, 11);
		model.add(WorldModel.DEPOT, 6, 9);
		model.add(WorldModel.DEPOT, 2, 3);
		for(int i = 0; i < 12; i++) model.add(WorldModel.OBSTACLE, i, 6);
		model.add(WorldModel.OBSTACLE, 2, 2);
		model.add(WorldModel.OBSTACLE, 5, 2);
		model.add(WorldModel.OBSTACLE, 8, 2);
		model.setInitialNbGolds(model.countObjects(WorldModel.GOLD));
		return model;
	}
    
    /** no gold/no obstacle world */
    static WorldModel world1() throws Exception {
        WorldModel model = WorldModel.create(21, 21, 4);
        model.setDepot(5, 7);
        model.setAgPos(0, 1, 0);
        model.setAgPos(1, 20, 0);
        model.setAgPos(2, 3, 20);
        model.setAgPos(3, 20, 20);
		model.add(WorldModel.OBSTACLE, 20, 19);
		model.add(WorldModel.OBSTACLE, 19, 19);
		model.add(WorldModel.OBSTACLE, 18, 19);
		model.add(WorldModel.OBSTACLE, 14, 14);
		model.add(WorldModel.OBSTACLE, 18, 7);
		model.add(WorldModel.OBSTACLE, 3, 10);
		model.add(WorldModel.OBSTACLE, 4, 7);
		model.add(WorldModel.OBSTACLE, 6, 7);
		model.add(WorldModel.OBSTACLE, 5, 13);
		model.add(WorldModel.OBSTACLE, 5, 5);
		model.add(WorldModel.OBSTACLE, 7, 18);
		model.add(WorldModel.OBSTACLE, 12, 5);
		model.add(WorldModel.OBSTACLE, 8, 9);
        model.setInitialNbGolds(model.countObjects(WorldModel.GOLD));
        return model;
    }

    /** world with gold, no obstacle */
    static WorldModel world2() throws Exception {
        WorldModel model = WorldModel.create(35, 25, 4);
        model.setId("Scenario 4");
		model.add(WorldModel.OBSTACLE, 7, 0);
		model.add(WorldModel.OBSTACLE, 12, 0);
		model.add(WorldModel.OBSTACLE, 22, 0);
		model.add(WorldModel.OBSTACLE, 7, 1);
		model.add(WorldModel.OBSTACLE, 12, 1);
		model.add(WorldModel.OBSTACLE, 22, 1);
		model.setAgPos(0, 2, 2);
		model.add(WorldModel.OBSTACLE, 7, 2);
		model.add(WorldModel.OBSTACLE, 12, 2);
		model.add(WorldModel.OBSTACLE, 7, 3);
		model.add(WorldModel.GOLD, 9, 3);
		model.add(WorldModel.OBSTACLE, 12, 3);
		model.add(WorldModel.GOLD, 18, 3);
		model.add(WorldModel.GOLD, 19, 3);
		model.add(WorldModel.GOLD, 20, 3);
		model.add(WorldModel.OBSTACLE, 7, 4);
		model.add(WorldModel.OBSTACLE, 12, 4);
		model.add(WorldModel.GOLD, 18, 4);
		model.add(WorldModel.GOLD, 19, 4);
		model.add(WorldModel.OBSTACLE, 7, 5);
		model.add(WorldModel.OBSTACLE, 12, 5);
		model.add(WorldModel.OBSTACLE, 7, 6);
		model.add(WorldModel.GOLD, 9, 6);
		model.add(WorldModel.OBSTACLE, 12, 6);
		model.add(WorldModel.OBSTACLE, 16, 6);
		model.add(WorldModel.OBSTACLE, 17, 6);
		model.add(WorldModel.OBSTACLE, 18, 6);
		model.add(WorldModel.OBSTACLE, 19, 6);
		model.add(WorldModel.OBSTACLE, 20, 6);
		model.add(WorldModel.OBSTACLE, 21, 6);
		model.add(WorldModel.OBSTACLE, 22, 6);
		model.add(WorldModel.OBSTACLE, 23, 6);
		model.add(WorldModel.OBSTACLE, 24, 6);
		model.add(WorldModel.OBSTACLE, 25, 6);
		model.add(WorldModel.OBSTACLE, 26, 6);
		model.add(WorldModel.OBSTACLE, 27, 6);
		model.add(WorldModel.OBSTACLE, 28, 6);
		model.add(WorldModel.OBSTACLE, 29, 6);
		model.add(WorldModel.OBSTACLE, 30, 6);
		model.add(WorldModel.OBSTACLE, 7, 7);
		model.add(WorldModel.OBSTACLE, 12, 7);
		model.add(WorldModel.OBSTACLE, 7, 8);
		model.add(WorldModel.OBSTACLE, 7, 9);
		model.add(WorldModel.OBSTACLE, 11, 9);
		model.add(WorldModel.OBSTACLE, 12, 9);
		model.add(WorldModel.OBSTACLE, 13, 9);
		model.add(WorldModel.OBSTACLE, 14, 9);
		model.add(WorldModel.OBSTACLE, 15, 9);
		model.add(WorldModel.OBSTACLE, 16, 9);
		model.add(WorldModel.OBSTACLE, 17, 9);
		model.add(WorldModel.OBSTACLE, 18, 9);
		model.add(WorldModel.OBSTACLE, 19, 9);
		model.add(WorldModel.OBSTACLE, 20, 9);
		model.add(WorldModel.OBSTACLE, 21, 9);
		model.add(WorldModel.OBSTACLE, 25, 9);
		model.add(WorldModel.OBSTACLE, 7, 10);
		model.add(WorldModel.OBSTACLE, 25, 10);
		model.add(WorldModel.OBSTACLE, 7, 11);
		model.setAgPos(1, 15, 11);
		model.add(WorldModel.OBSTACLE, 25, 11);
		model.add(WorldModel.OBSTACLE, 25, 12);
		model.add(WorldModel.OBSTACLE, 5, 13);
		model.add(WorldModel.OBSTACLE, 6, 13);
		model.add(WorldModel.OBSTACLE, 7, 13);
		model.add(WorldModel.OBSTACLE, 8, 13);
		model.add(WorldModel.OBSTACLE, 9, 13);
		model.add(WorldModel.OBSTACLE, 10, 13);
		model.add(WorldModel.OBSTACLE, 11, 13);
		model.add(WorldModel.OBSTACLE, 12, 13);
		model.add(WorldModel.OBSTACLE, 13, 13);
		model.add(WorldModel.OBSTACLE, 14, 13);
		model.add(WorldModel.OBSTACLE, 15, 13);
		model.add(WorldModel.OBSTACLE, 16, 13);
		model.add(WorldModel.OBSTACLE, 17, 13);
		model.add(WorldModel.OBSTACLE, 18, 13);
		model.add(WorldModel.OBSTACLE, 19, 13);
		model.add(WorldModel.OBSTACLE, 25, 13);
		model.add(WorldModel.OBSTACLE, 25, 14);
		model.add(WorldModel.OBSTACLE, 14, 15);
		model.add(WorldModel.OBSTACLE, 25, 15);
		model.add(WorldModel.OBSTACLE, 1, 16);
		model.add(WorldModel.OBSTACLE, 2, 16);
		model.add(WorldModel.OBSTACLE, 3, 16);
		model.add(WorldModel.OBSTACLE, 4, 16);
		model.add(WorldModel.OBSTACLE, 5, 16);
		model.add(WorldModel.OBSTACLE, 6, 16);
		model.add(WorldModel.OBSTACLE, 7, 16);
		model.add(WorldModel.OBSTACLE, 8, 16);
		model.add(WorldModel.OBSTACLE, 14, 16);
		model.add(WorldModel.GOLD, 24, 16);
		model.add(WorldModel.GOLD, 26, 16);
		model.setDepot(5, 17);
		model.add(WorldModel.OBSTACLE, 14, 17);
		model.add(WorldModel.OBSTACLE, 18, 17);
		model.add(WorldModel.OBSTACLE, 19, 17);
		model.add(WorldModel.OBSTACLE, 20, 17);
		model.add(WorldModel.OBSTACLE, 21, 17);
		model.add(WorldModel.OBSTACLE, 22, 17);
		model.add(WorldModel.OBSTACLE, 23, 17);
		model.add(WorldModel.OBSTACLE, 24, 17);
		model.add(WorldModel.OBSTACLE, 25, 17);
		model.add(WorldModel.OBSTACLE, 26, 17);
		model.add(WorldModel.OBSTACLE, 27, 17);
		model.add(WorldModel.OBSTACLE, 4, 18);
		model.add(WorldModel.OBSTACLE, 7, 18);
		model.add(WorldModel.OBSTACLE, 14, 18);
		model.add(WorldModel.GOLD, 3, 19);
		model.add(WorldModel.OBSTACLE, 4, 19);
		model.add(WorldModel.OBSTACLE, 7, 19);
		model.setAgPos(2, 11, 19);
		model.add(WorldModel.OBSTACLE, 14, 19);
		model.add(WorldModel.OBSTACLE, 4, 20);
		model.add(WorldModel.OBSTACLE, 7, 20);
		model.add(WorldModel.OBSTACLE, 14, 20);
		model.add(WorldModel.GOLD, 20, 20);
		model.add(WorldModel.GOLD, 21, 20);
		model.add(WorldModel.GOLD, 25, 20);
		model.add(WorldModel.GOLD, 26, 20);
		model.add(WorldModel.OBSTACLE, 14, 21);
		model.add(WorldModel.GOLD, 20, 21);
		model.add(WorldModel.GOLD, 21, 21);
		model.add(WorldModel.GOLD, 25, 21);
		model.add(WorldModel.GOLD, 26, 21);
		model.add(WorldModel.OBSTACLE, 3, 22);
		model.add(WorldModel.OBSTACLE, 4, 22);
		model.add(WorldModel.OBSTACLE, 5, 22);
		model.add(WorldModel.OBSTACLE, 6, 22);
		model.add(WorldModel.OBSTACLE, 7, 22);
		model.setAgPos(3, 23, 22);
		model.add(WorldModel.GOLD, 19, 23);
		model.add(WorldModel.GOLD, 20, 23);
		model.add(WorldModel.GOLD, 26, 23);
		model.add(WorldModel.GOLD, 27, 23);
		model.add(WorldModel.GOLD, 20, 24);
		model.add(WorldModel.GOLD, 21, 24);
		model.add(WorldModel.GOLD, 22, 24);
		model.add(WorldModel.GOLD, 23, 24);
		model.add(WorldModel.GOLD, 24, 24);
		model.add(WorldModel.GOLD, 25, 24);
		model.add(WorldModel.GOLD, 26, 24);        
        model.setInitialNbGolds(model.countObjects(WorldModel.GOLD));
        return model;
    }

    /** world with gold and obstacles */
    static WorldModel world3() throws Exception {
        WorldModel model = WorldModel.create(35, 35, 4);
        model.setId("Scenario 5");
        model.setDepot(16, 16);
        model.setAgPos(0, 1, 0);
        model.setAgPos(1, 20, 0);
        model.setAgPos(2, 6, 26);
        model.setAgPos(3, 20, 20);
        model.add(WorldModel.GOLD, 20, 13);
        model.add(WorldModel.GOLD, 15, 20);
        model.add(WorldModel.GOLD, 1, 1);
        model.add(WorldModel.GOLD, 3, 5);
        model.add(WorldModel.GOLD, 24, 24);
        model.add(WorldModel.GOLD, 20, 20);
        model.add(WorldModel.GOLD, 20, 21);
        model.add(WorldModel.GOLD, 2, 22);
        model.add(WorldModel.GOLD, 2, 12);
        model.add(WorldModel.GOLD, 19, 2);
        model.add(WorldModel.GOLD, 14, 4);
        model.add(WorldModel.GOLD, 34, 34);

        model.add(WorldModel.OBSTACLE, 12, 3);
        model.add(WorldModel.OBSTACLE, 13, 3);
        model.add(WorldModel.OBSTACLE, 14, 3);
        model.add(WorldModel.OBSTACLE, 15, 3);
        model.add(WorldModel.OBSTACLE, 18, 3);
        model.add(WorldModel.OBSTACLE, 19, 3);
        model.add(WorldModel.OBSTACLE, 20, 3);
        model.add(WorldModel.OBSTACLE, 14, 8);
        model.add(WorldModel.OBSTACLE, 15, 8);
        model.add(WorldModel.OBSTACLE, 16, 8);
        model.add(WorldModel.OBSTACLE, 17, 8);
        model.add(WorldModel.OBSTACLE, 19, 8);
        model.add(WorldModel.OBSTACLE, 20, 8);

        model.add(WorldModel.OBSTACLE, 12, 32);
        model.add(WorldModel.OBSTACLE, 13, 32);
        model.add(WorldModel.OBSTACLE, 14, 32);
        model.add(WorldModel.OBSTACLE, 15, 32);
        model.add(WorldModel.OBSTACLE, 18, 32);
        model.add(WorldModel.OBSTACLE, 19, 32);
        model.add(WorldModel.OBSTACLE, 20, 32);
        model.add(WorldModel.OBSTACLE, 14, 28);
        model.add(WorldModel.OBSTACLE, 15, 28);
        model.add(WorldModel.OBSTACLE, 16, 28);
        model.add(WorldModel.OBSTACLE, 17, 28);
        model.add(WorldModel.OBSTACLE, 19, 28);
        model.add(WorldModel.OBSTACLE, 20, 28);

        model.add(WorldModel.OBSTACLE, 3, 12);
        model.add(WorldModel.OBSTACLE, 3, 13);
        model.add(WorldModel.OBSTACLE, 3, 14);
        model.add(WorldModel.OBSTACLE, 3, 15);
        model.add(WorldModel.OBSTACLE, 3, 18);
        model.add(WorldModel.OBSTACLE, 3, 19);
        model.add(WorldModel.OBSTACLE, 3, 20);
        model.add(WorldModel.OBSTACLE, 8, 14);
        model.add(WorldModel.OBSTACLE, 8, 15);
        model.add(WorldModel.OBSTACLE, 8, 16);
        model.add(WorldModel.OBSTACLE, 8, 17);
        model.add(WorldModel.OBSTACLE, 8, 19);
        model.add(WorldModel.OBSTACLE, 8, 20);

        model.add(WorldModel.OBSTACLE, 32, 12);
        model.add(WorldModel.OBSTACLE, 32, 13);
        model.add(WorldModel.OBSTACLE, 32, 14);
        model.add(WorldModel.OBSTACLE, 32, 15);
        model.add(WorldModel.OBSTACLE, 32, 18);
        model.add(WorldModel.OBSTACLE, 32, 19);
        model.add(WorldModel.OBSTACLE, 32, 20);
        model.add(WorldModel.OBSTACLE, 28, 14);
        model.add(WorldModel.OBSTACLE, 28, 15);
        model.add(WorldModel.OBSTACLE, 28, 16);
        model.add(WorldModel.OBSTACLE, 28, 17);
        model.add(WorldModel.OBSTACLE, 28, 19);
        model.add(WorldModel.OBSTACLE, 28, 20);

        model.add(WorldModel.OBSTACLE, 13, 13);
        model.add(WorldModel.OBSTACLE, 13, 14);

        model.add(WorldModel.OBSTACLE, 13, 16);
        model.add(WorldModel.OBSTACLE, 13, 17);

        model.add(WorldModel.OBSTACLE, 13, 19);
        model.add(WorldModel.OBSTACLE, 14, 19);

        model.add(WorldModel.OBSTACLE, 16, 19);
        model.add(WorldModel.OBSTACLE, 17, 19);

        model.add(WorldModel.OBSTACLE, 19, 19);
        model.add(WorldModel.OBSTACLE, 19, 18);

        model.add(WorldModel.OBSTACLE, 19, 16);
        model.add(WorldModel.OBSTACLE, 19, 15);

        model.add(WorldModel.OBSTACLE, 19, 13);
        model.add(WorldModel.OBSTACLE, 18, 13);

        model.add(WorldModel.OBSTACLE, 16, 13);
        model.add(WorldModel.OBSTACLE, 15, 13);

        // labirinto
        model.add(WorldModel.OBSTACLE, 2, 32);
        model.add(WorldModel.OBSTACLE, 3, 32);
        model.add(WorldModel.OBSTACLE, 4, 32);
        model.add(WorldModel.OBSTACLE, 5, 32);
        model.add(WorldModel.OBSTACLE, 6, 32);
        model.add(WorldModel.OBSTACLE, 7, 32);
        model.add(WorldModel.OBSTACLE, 8, 32);
        model.add(WorldModel.OBSTACLE, 9, 32);
        model.add(WorldModel.OBSTACLE, 10, 32);
        model.add(WorldModel.OBSTACLE, 10, 31);
        model.add(WorldModel.OBSTACLE, 10, 30);
        model.add(WorldModel.OBSTACLE, 10, 29);
        model.add(WorldModel.OBSTACLE, 10, 28);
        model.add(WorldModel.OBSTACLE, 10, 27);
        model.add(WorldModel.OBSTACLE, 10, 26);
        model.add(WorldModel.OBSTACLE, 10, 25);
        model.add(WorldModel.OBSTACLE, 10, 24);
        model.add(WorldModel.OBSTACLE, 10, 23);
        model.add(WorldModel.OBSTACLE, 2, 23);
        model.add(WorldModel.OBSTACLE, 3, 23);
        model.add(WorldModel.OBSTACLE, 4, 23);
        model.add(WorldModel.OBSTACLE, 5, 23);
        model.add(WorldModel.OBSTACLE, 6, 23);
        model.add(WorldModel.OBSTACLE, 7, 23);
        model.add(WorldModel.OBSTACLE, 8, 23);
        model.add(WorldModel.OBSTACLE, 9, 23);
        model.add(WorldModel.OBSTACLE, 2, 29);
        model.add(WorldModel.OBSTACLE, 2, 28);
        model.add(WorldModel.OBSTACLE, 2, 27);
        model.add(WorldModel.OBSTACLE, 2, 26);
        model.add(WorldModel.OBSTACLE, 2, 25);
        model.add(WorldModel.OBSTACLE, 2, 24);
        model.add(WorldModel.OBSTACLE, 2, 23);
        model.add(WorldModel.OBSTACLE, 2, 29);
        model.add(WorldModel.OBSTACLE, 3, 29);
        model.add(WorldModel.OBSTACLE, 4, 29);
        model.add(WorldModel.OBSTACLE, 5, 29);
        model.add(WorldModel.OBSTACLE, 6, 29);
        model.add(WorldModel.OBSTACLE, 7, 29);
        model.add(WorldModel.OBSTACLE, 7, 28);
        model.add(WorldModel.OBSTACLE, 7, 27);
        model.add(WorldModel.OBSTACLE, 7, 26);
        model.add(WorldModel.OBSTACLE, 7, 25);
        model.add(WorldModel.OBSTACLE, 6, 25);
        model.add(WorldModel.OBSTACLE, 5, 25);
        model.add(WorldModel.OBSTACLE, 4, 25);
        model.add(WorldModel.OBSTACLE, 4, 26);
        model.add(WorldModel.OBSTACLE, 4, 27);
        model.setInitialNbGolds(model.countObjects(WorldModel.GOLD));
        return model;
    }

}
